import matplotlib as mpl
mpl.use('Agg')
mpl.interactive(False)

from matplotlib import *
import observations
from observations import *

# parameters
params = {'backend': 'PS',
          'axes.labelsize': 10,
          'font.size': 10,
          'xtick.labelsize': 8,
          'ytick.labelsize': 8,
          'text.usetex': False}

rcParams.update(params)

# Read configuration file.
parser = optparse.OptionParser(usage="%prog configuration_file")
(options, args) = parser.parse_args()

if not args:
    parser.error("A configuration file is required.")
if not os.path.isfile(args[0]):
    parser.error("Configuration file " + str(args[0]) + " not found.")

additional_content = [("location", "[release]", "String"), \
                      ("x_release", "[release]", "Float"), \
                      ("y_release", "[release]", "Float"), \
                      ("species", "[input]", "String"), \
                      ("station_file", "[input]", "String"), \
                      ("statistical_measures", "[statistics]", "StringList"), \
                      ("options_out", "[output]", "StringList"), \
                      ("plume_evolution_frequency", "[output]", "Int"), \
                      ("threshold", "[output]", "Float"), \
                      ("unit", "[output]", "String"), \
                      ("meas_min", "[output]", "Float"), \
                      ("directory_out", "[output]", "String"), \
                      ("extension_out", "[output]", "String")]

config = talos.Config(args[0], additional_content=additional_content)


# Reads the stations and the measurements.
read_stations = getattr(observations, "read_stations_"
                        + config.location.lower())
read_measurements = getattr(observations,
                            "read_measurements_" + config.location.lower())

stations = read_stations(config.station_file)
observ = read_measurements(config, stations)


# Read simulated concentrations.
origin = (config.t_min, config.y_min, config.x_min)
delta = (config.Delta_t, config.Delta_y, config.Delta_x)
shape = (config.Nt, config.Ny, config.Nx)

d = getd(config)

d_station = observation.get_simulated_at_stations(origin, delta, d[:, 0], stations)

sim_dates = observation.get_simulation_dates(origin[0], delta[0] / 3600., shape[0])

if hasattr(config, 't_range'):
    istart = 0
    while istart < len(sim_dates) and sim_dates[istart] < config.t_range[0]:
        istart += 1
    iend = istart + 1
    while iend < len(sim_dates) and sim_dates[iend] <= config.t_range[1]:
        iend += 1
    sim_dates = sim_dates[istart:iend]
    for i in range(len(d_station)):
        d_station[i] = d_station[i][istart:iend]

# Filter over simulation period. Remove negative values and useless stations.
if hasattr(config, 'meas_min'):
    meas_min = config.meas_min
else:
    meas_min = 0
obs_dates, obs, d_station_filter, stations_filter \
 = filter_obs(observ, sim_dates, d_station, stations, meas_min)

simul = get_averaged_concentrations(obs_dates, sim_dates, d_station_filter)

# Global arrays.
obs_list = [[obs[i][j] for j in range(len(obs[i]))] for i in range(len(obs))]

simul_list = [[simul[i][j] for j in range(len(simul[i]))] for i in range(len(simul))]

# filter obs_list, simul_list over filter_value.
filter_value = 0.
tmp_obs = [[obs_list[k][j] for j in range(len(obs_list[k])) \
            if obs_list[k][j] > filter_value \
            and simul_list[k][j] > filter_value] for k in range(len(obs_list))]
tmp_sim = [[simul_list[k][j] for j in range(len(obs_list[k])) \
            if obs_list[k][j] > filter_value \
            and simul_list[k][j] > filter_value] for k in range(len(obs_list))]

obs_list = tmp_obs
simul_list = tmp_sim

tmp = []
for i in range(len(obs_list)):
    tmp += obs_list[i]

obs_array = ravel(array(tmp, 'f'))

tmp = []
for i in range(len(simul_list)):
   tmp += simul_list[i]

simul_array = ravel(array(tmp, 'f'))


def get_save_directory(option_out):
    save_directory = config.directory_out + "/"
    if option_out == '1':
        save_directory += "bar/"
    elif option_out == '2':
        save_directory += "evolution/"
    if not os.path.exists(save_directory):
        os.makedirs(save_directory)
    return save_directory


##################################################
# 1) bar

if (config.options_out.count('1') == 1):
    bar_ind, bar_width = compute_observation_bar(obs_dates)

    for i in range(len(d_station_filter)):
        clf()
        fig = []
        b = numpy.array(obs[i]).ravel()
        fig.append(plot_date(date2num(sim_dates), d_station_filter[i], 'b-', lw=2))
        fig.append(bar(bar_ind[i], b, bar_width[i], color='r'))
        for j in range(len(fig[1])):
            fig[1][j].fill = 0
        dd = []
        for j in range(len(bar_ind[i])):
            dd.append(bar_ind[i][j] + bar_width[i][j] * 0.5)
        fig.append(plot_date(dd, simul[i], 'bD', markersize=10))
        setp(gca().get_xticklabels(), fontsize=8)
        setp(gca().get_yticklabels(), fontsize=16)
        if hasattr(config, 't_range'):
            limits = axis()
            axis([date2num(config.t_range[0]), date2num(config.t_range[1]), limits[2], limits[3]])
        title(stations_filter[i].name)

        # english
        legend(('Simulation', 'Simulation averaged over\n measurement periods', 'Measurements'))
        ylabel('Volumic activity [' + config.unit + ']')

        draw()
        savefig(get_save_directory('1') + stations_filter[i].name + "." + config.extension_out)


################################################
# 2) Evolution maps

if (config.options_out.count('2') == 1):
    x, y = get_surface_coordinates(config)

    palette = ['#CCFFFF', '#99DDDD', '#66BBBB', '#339999', '#006666']

    levels = []
    num_max = int(log10(d.max()))
    color_max = len(palette)
    for i in range(color_max + 1):
        levels.append(pow(10., num_max - i + 1))

    levels.reverse()

    frequency = config.plume_evolution_frequency

    m = get_map(config)

    for i in range(int(len(sim_dates)) / frequency):
        conc = d[i * frequency, 0]
        if conc.max() > levels[0]:
            clf()
            m.drawcountries()
            m.drawcoastlines()
            m.plot([config.x_release], [config.y_release], 'r^', markersize=10)
            text(config.x_release, config.y_release + 1, config.location, \
                 horizontalalignment='center', \
                 verticalalignment='top', fontsize=10, color='r')
            m.contourf(x, y, conc, array(levels), colors=palette)
            date = sim_dates[i * frequency].isoformat()
            title(date)
            cax = axes_label(levels, palette)

            draw()
            savefig(get_save_directory('2') + date + "." \
                    + config.extension_out)


###################################################
# 3) Stations map.

if (config.options_out.count('3') == 1):
    clf()
    m = getm(config, cbar=False)
    m.drawcountries()
    m.drawcoastlines()
    m.fillcontinents()

    maxmes = [max(obs[i]) for i in range (len(obs))]
    maxmax = max(maxmes)

    for i in range(0, len(stations_filter)):
        xpt, ypt = m(stations_filter[i].longitude, stations_filter[i].latitude)
        size = 10 * sqrt(maxmes[i] / maxmax)
        m.plot([xpt], [ypt], 'bo', markersize=size)

    m.plot([config.x_release], [config.y_release], 'r^', markersize=10)
    text(config.x_release, config.y_release + 1, config.location, \
         horizontalalignment='center', verticalalignment='top', \
         fontsize=10, color='r')

    savefig(get_save_directory('3') + "station_map." + config.extension_out)


################################################
# 4) Statistical measures.

# Formats.
formats_nb = {"mnbe": "%.4g", "mnb": "%.4g", "nme": "%.4g", "mnge": "%.4g",
              "upa": "%.4g", "correlation": "%.4g", "determination": "%.4g",
              "epa": "%.4g", "rmse": "%.4g", "meas_mean": "%.4g",
              "sim_mean": "%.4g"}
formats_ratio = {"mnbe": 100., "mnb": 100., "nme": 100., "mnge": 100.,
                 "upa": 100., "correlation": 100., "determination": 100.,
                 "epa": 100.}
formats_end = {"mnbe": '%', "mnb": '%', "nme": '%', "mnge": '%', "upa": '%',
               "correlation": '%', "determination": '%', "epa": '%'}

def format(meas, nb, format_nb="%.4g"):
    format_ratio = 1.
    format_end = ''
    meas = meas.lower()
    if meas in globals()["formats_ratio"].keys():
        format_ratio = globals()["formats_ratio"][meas]
    if meas in globals()["formats_end"].keys():
        format_end = globals()["formats_end"][meas]
    return format_nb % (format_ratio * nb) + format_end

if (config.options_out.count('4') == 1):

    output_file = open(get_save_directory('4') + "/statistics.txt", 'w')
    talos.print_stdout_file(config.species + ' concentrations\n', output_file)

    mean_obs = zeros(len(stations_filter), 'f')
    mean_simul = zeros(len(stations_filter), 'f')
    for k in range(len(stations_filter)):
        if len(obs_list[k]) > 0:
            a = ravel(array(obs_list[k]))
            b = ravel(array(simul_list[k]))
            mean_obs[k] = a.mean()
            mean_simul[k] = b.mean()

            talos.print_stdout_file('\n', output_file)

            talos.print_stdout_file(str(k) + "\t" + stations_filter[k].name, \
                                    output_file)
            talos.print_stdout_file('Number of measures: ' \
                                    + str(len(obs_list[k])), output_file)
            talos.print_stdout_file('measurements mean: ' \
                                    + format('meas_mean', mean_obs[k]), \
                                    output_file)
            talos.print_stdout_file('simulation mean  : ' \
                                    + format('sim_mean', mean_simul[k]), \
                                    output_file)

            output = []
            if len(obs_list[k]) > 1 and mean_obs[k] != 0 and mean_simul[k] > 1e-8 :
                for meas in config.statistical_measures:
                    output.append(getattr(stat, meas.lower())(a, b))

                results = array(output)

                for i in range(len(config.statistical_measures)):
                    meas = config.statistical_measures[i]
                    talos.print_stdout_file(" \ " + meas + ": " \
                                            + format(meas, results[i]),
                                            output_file)

    talos.print_stdout_file('\n', output_file)
    talos.print_stdout_file(' ** Global **', output_file)
    talos.print_stdout_file('\n', output_file)
    talos.print_stdout_file('Number of measures: ' + str(len(obs_array)), \
                            output_file)
    talos.print_stdout_file('measurements mean: ' \
                            + format('meas_mean', obs_array.mean()), \
                            output_file)
    talos.print_stdout_file('simulation mean  : ' \
                            + format('sim_mean', simul_array.mean()), \
                            output_file)

    output = []
    for meas in config.statistical_measures:
        output.append(getattr(stat, meas.lower())(simul_array, obs_array))

    results = array(output)

    for i in range(len(config.statistical_measures)):
        meas = config.statistical_measures[i]
        talos.print_stdout_file(" \ " + meas + ": " \
                                + format(meas, results[i]),
                                output_file)

    output_file.close()

##################################################
# 5) scatter

if (config.options_out.count('5') == 1):
    clf()
    obs_scatter = []
    simul_scatter = []
    for i in range(len(obs_array)):
        if obs_array[i] > config.threshold \
               and simul_array[i] > config.threshold:
            obs_scatter.append(obs_array[i])
            simul_scatter.append(simul_array[i])

    scatter(obs_scatter, simul_scatter, s=5)
    gca().set_xscale('log')
    gca().set_yscale('log')

    limits = axis()
    lim_min = min(limits[0], limits[2])
    lim_min = min(config.threshold, lim_min)
    lim_max = min(limits[1], limits[3])

    plot([lim_min, lim_max], [lim_min, lim_max], 'k--')
    l = 10.
    plot([lim_min, lim_max], \
         [lim_min * l, lim_max * l], 'k:')
    plot([lim_min, lim_max], \
         [lim_min / l, lim_max / l], 'k:')

    axis([lim_min, lim_max, lim_min, lim_max])
    xlabel('Measurements  [' + config.unit + ']')
    ylabel('Polyphemus [' + config.unit + ']')
    title(config.species)
    savefig(get_save_directory('5') + "scatter_stations." + config.extension_out)
